"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const api_1 = require("../support/api");
const kubernetes_1 = require("../typings/kubernetes");
let KubeUndeployParameters = class KubeUndeployParameters {
    constructor() {
        this.ns = kubernetes_1.defaultNamespace;
    }
};
__decorate([
    automation_client_1.Parameter({
        displayName: "Name",
        description: "name of resources to remove",
        pattern: /^[a-z](?:[-a-z0-9]*[a-z0-9])?$/,
        validInput: "a valid Kubernetes resource name, beginning with a lowercase letter, ending with a lowercase" +
            "letter or number, and containing only lowercase letters, numbers, and dashes (-)",
        minLength: 1,
        maxLength: 63,
        required: true,
    }),
    __metadata("design:type", String)
], KubeUndeployParameters.prototype, "name", void 0);
__decorate([
    automation_client_1.Parameter({
        displayName: "Namespace",
        description: "namespace of resources to remove",
        pattern: /^[a-z](?:[-a-z0-9]*[a-z0-9])?$/,
        validInput: "a valid Kubernetes namespace, beginning with a lowercase letter, ending with a lowercase" +
            "letter or number, and containing only lowercase letters, numbers, and dashes (-)",
        minLength: 1,
        maxLength: 63,
        required: false,
    }),
    __metadata("design:type", String)
], KubeUndeployParameters.prototype, "ns", void 0);
__decorate([
    automation_client_1.Parameter({
        displayName: "Port",
        description: "port the application listens on, if not provided no service resources are removed",
        pattern: /^\d{1,5}$/,
        validInput: "a number between 1 and 65535, inclusive",
        minLength: 1,
        maxLength: 5,
        required: false,
    }),
    __metadata("design:type", String)
], KubeUndeployParameters.prototype, "port", void 0);
__decorate([
    automation_client_1.Parameter({
        displayName: "Path",
        description: "ingress path for the resource to remove, if not provided no service or ingress " +
            "resources are removed",
        pattern: /^\/\S+$/,
        validInput: "an asbolute URL path, unique for this Kubernetes cluster",
        minLength: 1,
        maxLength: 512,
        required: false,
    }),
    __metadata("design:type", String)
], KubeUndeployParameters.prototype, "path", void 0);
__decorate([
    automation_client_1.Parameter({
        displayName: "Host",
        description: "ingress hostname for the resources to remove, if not provided the rule without a host " +
            "is modified",
        pattern: /^[a-z0-9](?:[-a-z0-9]*[a-z0-9])?(?:\.[a-z0-9](?:[-a-z0-9]*[a-z0-9])?)*$/,
        validInput: "a valid hostname, each label beginning and ending with a lowercase letter or number and " +
            "containing only lowercase letters, numbers, and dashes (-), separated by periods (.)",
        minLength: 1,
        maxLength: 253,
        required: false,
    }),
    __metadata("design:type", String)
], KubeUndeployParameters.prototype, "host", void 0);
KubeUndeployParameters = __decorate([
    automation_client_1.Parameters()
], KubeUndeployParameters);
exports.KubeUndeployParameters = KubeUndeployParameters;
/**
 * Safely remove all resources related to an Kubernetes deployment.
 */
exports.kubeUndeploy = {
    name: "KubeUndeploy",
    intent: "kube undeploy",
    description: "remove all resources related to an application from Kubernetes cluster",
    paramsMaker: KubeUndeployParameters,
    listener: (ci) => __awaiter(this, void 0, void 0, function* () {
        let k8Config;
        try {
            k8Config = api_1.getKubeConfig();
        }
        catch (e) {
            return ci.context.messageClient.respond(e.message)
                .then(() => ({ code: automation_client_1.Failure.code, message: e.message }), err => {
                const msg = `Failed to send response message: ${err.message}`;
                return { code: automation_client_1.Failure.code, message: `${e.message}; ${msg}` };
            });
        }
        const req = {
            config: k8Config,
            name: ci.parameters.name,
            ns: ci.parameters.ns,
            path: ci.parameters.path,
            host: ci.parameters.host,
        };
        return api_1.deleteApplication(req)
            .then(() => {
            const message = `Successfully removed ${req.ns}/${req.name} resources from Kubernetes`;
            automation_client_1.logger.info(message);
            return ci.context.messageClient.respond(sdm_1.slackSuccessMessage("Kubernetes Undeploy", message))
                .then(() => automation_client_1.Success, err => {
                const msg = `Failed to send response message: ${err.message}`;
                automation_client_1.logger.error(msg);
                return { code: automation_client_1.Success.code, message: `${message} but ${msg}` };
            });
        }, e => {
            const message = `Failed to remove ${req.ns}/${req.name} resources from Kubernetes: ${e.message}`;
            automation_client_1.logger.error(message);
            return ci.context.messageClient.respond(message)
                .then(() => ({ code: automation_client_1.Failure.code, message }), err => {
                automation_client_1.logger.error(`Failed to send response message: ${err.message}`);
                return { code: automation_client_1.Failure.code, message: `${message}; ${err.message}` };
            });
        });
    }),
};
//# sourceMappingURL=kubeUndeploy.js.map