import { OnEvent } from "@atomist/automation-client";
import { EventHandlerRegistration, SdmGoalEvent } from "@atomist/sdm";
import { KubernetesApplicationOptions } from "../support/options";
import { KubeDeployRequestedSdmGoal } from "../typings/types";
export interface CommitForSdmGoal {
    image?: {
        imageName?: string;
    };
}
export declare class KubernetesDeployParameters {
    environment: string;
    /** cluster or namespace mode, default is cluster */
    mode: "cluster" | "namespace";
    namespaces: string[];
}
export declare const KubernetesDeploy: OnEvent<KubeDeployRequestedSdmGoal.Subscription, KubernetesDeployParameters>;
export declare const kubernetesDeploy: EventHandlerRegistration<KubeDeployRequestedSdmGoal.Subscription, KubernetesDeployParameters>;
/**
 * Determine if SDM goal event should trigger a deployment to
 * Kubernetes.
 *
 * @param g SDM goal event
 * @return Success if eligible, Failure if not, with message properly populated
 */
export declare function eligibleDeployGoal(goal: SdmGoalEvent): Promise<boolean>;
/**
 * Verify that the deployment of the application should be fulfilled
 * by this SDM.  Ensures the that environment and namespace of the
 * application and deployment parameters match, as appropriate.
 *
 * If deployment environment is not set, it is presumed to satisfy all
 * environments so any application environment is considered to match.
 *
 * If the deployment mode is "namespace", then the `POD_NAMESPACE`
 * environment variable must be defined.  If the values of `app.ns`
 * and `POD_NAMESPACE` environment variable are the same, it is a
 * match.  If they are not equal, `undefined` will be returned.  If
 * `POD_NAMESPACE` is not set, an `Error` is thrown.
 *
 * If the deployment mode is "cluster" or not set and the deployment
 * namespaces array has elements, then then the value of `app.ns` must
 * be in the array for there to be a match.  If the deployment
 * namespaces array is not set or zero length, any value of `app.ns`
 * is considered a match.
 *
 * @param app Kubernetes application options data from SDM goal.
 * @param deploy Kubernetes deployment parameters from command or configuration.
 * @return verified Kubernetes application options if deployment should proceed, `undefined` otherwise.
 */
export declare function verifyKubernetesApplicationDeploy(app: KubernetesApplicationOptions, deploy: KubernetesDeployParameters): KubernetesApplicationOptions | undefined;
