"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const sdm_1 = require("@atomist/sdm");
const appRoot = require("app-root-path");
const fs = require("fs-extra");
const stringify = require("json-stringify-safe");
const path = require("path");
const api_1 = require("../support/api");
const deploy_1 = require("../support/deploy");
const kubernetes_1 = require("../typings/kubernetes");
let KubernetesDeployParameters = class KubernetesDeployParameters {
    constructor() {
        /** cluster or namespace mode, default is cluster */
        this.mode = "cluster";
    }
};
__decorate([
    automation_client_1.Value("environment"),
    __metadata("design:type", String)
], KubernetesDeployParameters.prototype, "environment", void 0);
__decorate([
    automation_client_1.Value({
        path: "sdm.k8.mode",
        required: false,
    }),
    __metadata("design:type", String)
], KubernetesDeployParameters.prototype, "mode", void 0);
__decorate([
    automation_client_1.Value({
        path: "sdm.k8.namespaces",
        required: false,
    }),
    __metadata("design:type", Array)
], KubernetesDeployParameters.prototype, "namespaces", void 0);
KubernetesDeployParameters = __decorate([
    automation_client_1.Parameters()
], KubernetesDeployParameters);
exports.KubernetesDeployParameters = KubernetesDeployParameters;
/* tslint:disable:cyclomatic-complexity */
exports.KubernetesDeploy = (ef, ctx, params) => __awaiter(this, void 0, void 0, function* () {
    if (!ef.data.SdmGoal) {
        automation_client_1.logger.warn(`Received event had no SdmGoal`);
        return Promise.resolve(automation_client_1.Success);
    }
    return Promise.all(ef.data.SdmGoal.map((g) => __awaiter(this, void 0, void 0, function* () {
        const sdmGoal = g;
        const eligible = yield eligibleDeployGoal(sdmGoal);
        if (!eligible) {
            automation_client_1.logger.info("SDM goal is not eligible for Kubernetes deploy");
            return automation_client_1.Success;
        }
        const repo = g.repo.name;
        const owner = g.repo.owner;
        const sha = g.sha;
        const workspaceId = ctx.workspaceId;
        const env = params.environment;
        const depName = `${workspaceId}:${env}:${owner}:${repo}:${sha}`;
        const commit = yield sdm_1.fetchCommitForSdmGoal(ctx, sdmGoal);
        if (!commit.image) {
            const msg = `Kubernetes deploy requested for ${depName} but that commit ` +
                `has no Docker image associated with it`;
            return failGoal(ctx, sdmGoal, msg);
        }
        const image = commit.image.imageName;
        automation_client_1.logger.debug(`Processing ${depName}`);
        let k8Config;
        try {
            k8Config = api_1.getKubeConfig();
        }
        catch (e) {
            return failGoal(ctx, sdmGoal, e.message);
        }
        let kubeGoalData = deploy_1.validateSdmGoal(sdmGoal);
        if (!kubeGoalData) {
            automation_client_1.logger.debug(`No Kubernetes goal data found for ${depName}`);
            return automation_client_1.Success;
        }
        try {
            kubeGoalData = verifyKubernetesApplicationDeploy(kubeGoalData, params);
            if (!kubeGoalData) {
                automation_client_1.logger.debug(`Kubernetes deployment data did not match parameters for ${depName}`);
                return automation_client_1.Success;
            }
        }
        catch (e) {
            const msg = `${depName} ${e.message}`;
            return failGoal(ctx, sdmGoal, msg);
        }
        const kubeApp = Object.assign({}, kubeGoalData, { workspaceId,
            image, ns: kubeGoalData.ns || kubernetes_1.defaultNamespace });
        automation_client_1.logger.info(`Deploying ${depName} to Kubernetes`);
        const upsertReq = Object.assign({}, kubeApp, { config: k8Config });
        try {
            yield api_1.upsertApplication(upsertReq);
        }
        catch (e) {
            const msg = `Failed to deploy ${depName} to Kubernetes: ${e.message}`;
            return failGoal(ctx, sdmGoal, msg);
        }
        automation_client_1.logger.info(`Successfully deployed ${depName} to Kubernetes`);
        const upParams = {
            state: sdm_1.SdmGoalState.success,
            description: `Deployed to Kubernetes namespace \`${kubeApp.ns}\``,
        };
        if (kubeApp.path && kubeApp.host) {
            const label = `Kubernetes ${env}:${kubeApp.ns}`;
            const url = api_1.endpointBaseUrl(kubeApp);
            upParams.externalUrls = [{ label, url }];
        }
        try {
            yield sdm_1.updateGoal(ctx, sdmGoal, upParams);
        }
        catch (e) {
            const msg = `Successfully deployed ${depName} to Kubernetes, but failed to ` +
                `update the SDM goal: ${e.message}`;
            return failGoal(ctx, sdmGoal, msg);
        }
        return automation_client_1.Success;
    })))
        .then(automation_client_1.reduceResults);
});
/* tslint:enable:cyclomatic-complexity */
exports.kubernetesDeploy = {
    name: "KubeDeploy",
    description: "Deploy application resources to Kubernetes cluster",
    tags: ["deploy", "kubernetes"],
    subscription: automation_client_1.GraphQL.subscription("KubeDeployRequestedSdmGoal"),
    paramsMaker: KubernetesDeployParameters,
    listener: exports.KubernetesDeploy,
};
/**
 * Determine if SDM goal event should trigger a deployment to
 * Kubernetes.
 *
 * @param g SDM goal event
 * @return Success if eligible, Failure if not, with message properly populated
 */
function eligibleDeployGoal(goal) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!goal.fulfillment) {
            automation_client_1.logger.debug(`SDM goal contains no fulfillment: ${stringify(goal)}`);
            return false;
        }
        const pkgPath = path.join(appRoot.path, "package.json");
        try {
            const pkg = yield fs.readJson(pkgPath);
            if (goal.fulfillment.name !== pkg.name) {
                automation_client_1.logger.debug(`SDM goal fulfillment name '${goal.fulfillment.name}' is not '${pkg.name}'`);
                return false;
            }
        }
        catch (e) {
            automation_client_1.logger.warn(`Failed to determine package name: ${e.message}`);
            return false;
        }
        const atmMethod = "side-effect";
        if (goal.fulfillment.method !== atmMethod) {
            automation_client_1.logger.debug(`SDM goal fulfillment method '${goal.fulfillment.method}' is not '${atmMethod}'`);
            return false;
        }
        if (goal.state !== "requested") {
            automation_client_1.logger.debug(`SDM goal state '${goal.state}' is not 'requested'`);
            return false;
        }
        return true;
    });
}
exports.eligibleDeployGoal = eligibleDeployGoal;
/* tslint:disable:cyclomatic-complexity */
/**
 * Verify that the deployment of the application should be fulfilled
 * by this SDM.  Ensures the that environment and namespace of the
 * application and deployment parameters match, as appropriate.
 *
 * If deployment environment is not set, it is presumed to satisfy all
 * environments so any application environment is considered to match.
 *
 * If the deployment mode is "namespace", then the `POD_NAMESPACE`
 * environment variable must be defined.  If the values of `app.ns`
 * and `POD_NAMESPACE` environment variable are the same, it is a
 * match.  If they are not equal, `undefined` will be returned.  If
 * `POD_NAMESPACE` is not set, an `Error` is thrown.
 *
 * If the deployment mode is "cluster" or not set and the deployment
 * namespaces array has elements, then then the value of `app.ns` must
 * be in the array for there to be a match.  If the deployment
 * namespaces array is not set or zero length, any value of `app.ns`
 * is considered a match.
 *
 * @param app Kubernetes application options data from SDM goal.
 * @param deploy Kubernetes deployment parameters from command or configuration.
 * @return verified Kubernetes application options if deployment should proceed, `undefined` otherwise.
 */
function verifyKubernetesApplicationDeploy(app, deploy) {
    app.ns = app.ns || kubernetes_1.defaultNamespace;
    if (!deploy) {
        return app;
    }
    if (deploy.environment && deploy.environment !== app.environment) {
        automation_client_1.logger.debug(`SDM goal data kubernetes environment '${app.environment}' is not this ` +
            `environment '${deploy.environment}'`);
        return undefined;
    }
    if (deploy.mode === "namespace") {
        const podNs = process.env.POD_NAMESPACE;
        if (!podNs) {
            throw new Error(`Kubernetes deploy requested but k8-automation is running in ` +
                `namespace-scoped mode and the POD_NAMESPACE environment variable is not set`);
        }
        if (app.ns !== podNs) {
            automation_client_1.logger.info(`SDM goal data kubernetes namespace '${app.ns}' is not the name as ` +
                `k8-automation running in namespace-scoped mode '${podNs}'`);
            return undefined;
        }
    }
    else if (deploy.namespaces && deploy.namespaces.length > 0 && !deploy.namespaces.includes(app.ns)) {
        automation_client_1.logger.debug(`SDM goal data kubernetes namespace '${app.ns}' is not in managed ` +
            `namespaces '${deploy.namespaces.join(",")}'`);
        return undefined;
    }
    return app;
}
exports.verifyKubernetesApplicationDeploy = verifyKubernetesApplicationDeploy;
/* tslint:enable:cyclomatic-complexity */
/**
 * Fail the provided goal using the message to set the description and
 * error message.
 *
 * @param ctx handler context to use to send the update
 * @param goal SDM goal to update
 * @param message informative error message
 * @return a failure handler result using the provided error message
 */
function failGoal(ctx, goal, message) {
    automation_client_1.logger.error(message);
    const params = {
        state: sdm_1.SdmGoalState.failure,
        description: message,
        error: new Error(message),
    };
    return sdm_1.updateGoal(ctx, goal, params)
        .then(() => ({ code: 1, message }), err => {
        const msg = `Failed to update SDM goal '${stringify(goal)}' with params ` +
            `'${stringify(params)}': ${err.message}`;
        automation_client_1.logger.error(msg);
        return { code: 2, message: `${message}; ${msg}` };
    });
}
//# sourceMappingURL=kubeDeploy.js.map