import { GitProject } from "@atomist/automation-client";
import { FulfillableGoalDetails, FulfillableGoalWithRegistrations, FulfillmentRegistration, Goal, RepoContext, SdmGoalEvent, SoftwareDeliveryMachine } from "@atomist/sdm";
import { Deployment, Service } from "../typings/kubernetes";
import { KubernetesApplicationOptions } from "./options";
/**
 * Registration to pass to KubernetesDeploy goal to describe the deployment metadata
 */
export interface KubernetesDeployRegistration extends FulfillmentRegistration {
    /**
     * Create the raw deployment data (name, ns, environment, ingress etc)
     */
    deploymentData?: (goal: SdmGoalEvent, context: RepoContext) => Promise<KubernetesApplicationOptions>;
    /**
     * Create the service spec patch for this application
     */
    serviceSpecCreator?: (serviceSpec: Service, goal: SdmGoalEvent, context: RepoContext) => Promise<Service>;
    /**
     * Create the deployment spec path
     */
    deploymentSpecCreator?: (deploymentSpec: Deployment, goal: SdmGoalEvent, context: RepoContext) => Promise<Deployment>;
}
/**
 * Goal that deploys an application to a Kubernetes cluster by using a direct deploy in SDM local mode or
 * triggers deploy via k8-automation for non-local mode use.
 */
export declare class KubernetesDeploy extends FulfillableGoalWithRegistrations<KubernetesDeployRegistration> {
    readonly details?: {
        environment: "testing" | "production" | string;
    } & FulfillableGoalDetails;
    constructor(details?: {
        environment: "testing" | "production" | string;
    } & FulfillableGoalDetails, ...dependsOn: Goal[]);
    /**
     * Register a deployment with all required callbacks
     */
    with(registration: KubernetesDeployRegistration): this;
    /**
     * Convenience method to register a deployment
     */
    withDeployment(deploymentData?: (goal: SdmGoalEvent, context: RepoContext) => Promise<KubernetesApplicationOptions>, serviceSpecCreator?: (serviceSpec: Service, goal: SdmGoalEvent, context: RepoContext) => Promise<Service>, deploymentSpecCreator?: (deploymentSpec: Deployment, goal: SdmGoalEvent, context: RepoContext) => Promise<Deployment>): this;
    /**
     * Called by the SDM on initialization
     */
    register(sdm: SoftwareDeliveryMachine): void;
}
/**
 * Default deployment data callback that reads the Dockerfile and uses EXPOSE instructions to setup
 * ingress rules.
 */
export declare function defaultDeploymentData(p: GitProject, goal: SdmGoalEvent, ctx: RepoContext, k8Deploy: KubernetesDeploy): Promise<KubernetesApplicationOptions>;
