import { GitProject, Project } from "@atomist/automation-client";
import { SdmGoalEvent } from "@atomist/sdm";
import { KubernetesApplicationOptions } from "./options";
/**
 * Subset of KubeApplication properties, omitting those that are
 * typically gotten from the context or build.  See the
 * [[KubeApplication]] documentation for property documentation.
 */
export interface KubernetesGoalData extends KubernetesApplicationOptions {
    deploymentSpec?: string;
    serviceSpec?: string;
}
/**
 * Adds Kubernetes application data to an SDM goal.  The objects
 * written under the kubernetes property is of type
 * [[KubeApplicationSpec]].
 *
 * @param goal SDM goal to add Kubernetes data to
 * @param options Kubernetes application options to set on the goal
 * @param p Project this goal is operating on
 * @returns SDM goal with Kuberenetes application properties set
 */
export declare function createKubernetesData(goal: SdmGoalEvent, options: KubernetesApplicationOptions, p: GitProject): Promise<SdmGoalEvent>;
/**
 * Reads Kubernetes deployment.json and service.json specs from the
 * project's .atomist/kubernetes folder.  It swallows all exceptions,
 * returning undefined if one occurs.
 *
 * @param p Project to look for spec file in
 * @param name File name of spec under .atomist/kubernetes
 * @returns a string if the spec was successfully read, undefined otherwise
 */
export declare function readKubernetesSpec(p: Project, name: string): Promise<string | undefined>;
