"use strict";
/*
 * Copyright © 2018 Atomist, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const automation_client_1 = require("@atomist/automation-client");
const path = require("path");
/**
 * Adds Kubernetes application data to an SDM goal.  The objects
 * written under the kubernetes property is of type
 * [[KubeApplicationSpec]].
 *
 * @param goal SDM goal to add Kubernetes data to
 * @param options Kubernetes application options to set on the goal
 * @param p Project this goal is operating on
 * @returns SDM goal with Kuberenetes application properties set
 */
function createKubernetesData(goal, options, p) {
    return __awaiter(this, void 0, void 0, function* () {
        const deploymentSpec = yield readKubernetesSpec(p, "deployment.json");
        const serviceSpec = yield readKubernetesSpec(p, "service.json");
        const kubernetesGoalData = Object.assign({}, options, { deploymentSpec,
            serviceSpec });
        return Object.assign({}, goal, { data: JSON.stringify({ kubernetes: kubernetesGoalData }) });
    });
}
exports.createKubernetesData = createKubernetesData;
/**
 * Reads Kubernetes deployment.json and service.json specs from the
 * project's .atomist/kubernetes folder.  It swallows all exceptions,
 * returning undefined if one occurs.
 *
 * @param p Project to look for spec file in
 * @param name File name of spec under .atomist/kubernetes
 * @returns a string if the spec was successfully read, undefined otherwise
 */
function readKubernetesSpec(p, name) {
    return __awaiter(this, void 0, void 0, function* () {
        const specPath = path.join(".atomist", "kubernetes", name);
        try {
            const specFile = yield p.getFile(specPath);
            if (specFile) {
                const spec = specFile.getContent();
                return spec;
            }
        }
        catch (e) {
            automation_client_1.logger.warn(`Failed to read spec file ${specPath}: ${e.message}`);
        }
        return undefined;
    });
}
exports.readKubernetesSpec = readKubernetesSpec;
//# sourceMappingURL=goal.js.map